"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class UnaryExpression {
    static visit(node, visitor) {
        const operator = node.operator;
        switch (operator) {
            case '-': {
                const arg = visitor.visitNode(node.argument);
                return -arg;
            }
            case '+': {
                const arg = visitor.visitNode(node.argument);
                return +arg;
            }
            case '!': {
                const arg = visitor.visitNode(node.argument);
                return !arg;
            }
            case '~': {
                const arg = visitor.visitNode(node.argument);
                return ~arg;
            }
            case 'void': {
                visitor.visitNode(node.argument);
                return undefined;
            }
            case 'typeof': {
                const arg = visitor.visitNode(node.argument);
                return typeof arg;
            }
            case 'delete': {
                if (node.argument.type === 'MemberExpression') {
                    const obj = visitor.visitNode(node.argument.object);
                    const prop = node.argument.computed ? visitor.visitNode(node.argument.property) : visitor.getName(node.argument.property);
                    return delete obj[prop];
                }
                else if (node.argument.type === 'Identifier') {
                    return visitor.scope.delete(node.argument.name);
                }
                return true;
            }
            default:
                console.warn('Unhandled UnaryExpression operator', operator);
        }
    }
}
exports.default = UnaryExpression;
